#include "she.h"
#include "cy_device_headers.h"
#include "cy_project.h"

#include "sysclk/cy_sysclk.h"
#include "gpio/cy_gpio.h"
#include "tcpwm/cy_tcpwm_pwm.h"
#include "trigmux/cy_trigmux.h"
#include "dma/cy_pdma.h"


#define TIMER_CTRL_COUNT_UP             0x00u
#define TIMER_CTRL_COUNT_DOWN           0x01u
#define TIMER_CTRL_COUNT_UPDN1          0x02u
#define TIMER_CTRL_COUNT_UPDN2          0x03u

#define TIMER_TR_OUT_ON_OVERFLOW        0x00u    /* Trigger out on overflow*/
#define TIMER_TR_OUT_ON_UNDERFLOW       0x01u    /* Trigger out on underflow*/
#define TIMER_TR_OUT_ON_TC              0x02u    /* Trigger out on terminal count*/
#define TIMER_TR_OUT_ON_CC0             0x03u    /* Trigger out on cc1 event*/
#define TIMER_TR_OUT_ON_CC1             0x04u    /* Trigger out on cc1 event*/
#define TIMER_TR_OUT_DISABLED           0x07u    /* Trigger out disabled*/

/* Defines for inverting PWM out and compl_out */
#define TIMER_CTRL_INV_OUT              0x01u
#define TIMER_CTRL_INV_COMPL_OUT        0x02u
#define TIMER_CTRL_INV_BOTH_OUT         0x03u

/* PWM PR Mode Configuration def */
#define TCPWM0_GRPx_CNTx_PWM            TCPWM0_GRP1_CNT6
#define PCLK_TCPWM0_CLOCKSx_PWM         PCLK_TCPWM0_CLOCKS262
#define TCPWM_PERI_CLK_DIVIDER_NO_PWM   0u

/* TCPWM_LINE0 */
#define TCPWM_LINEx_PORT                GPIO_PRT18
#define TCPWM_LINEx_PIN                 0u
#define TCPWM_LINEx_MUX                 P18_0_TCPWM0_LINE262

/* TCPWM_LINE1 */
#define TCPWM_LINE_COMPLx_PORT          GPIO_PRT18
#define TCPWM_LINE_COMPLx_PIN           1u
#define TCPWM_LINE_COMPLx_MUX           P18_1_TCPWM0_LINE_COMPL262

/* DMA chanells */
#define DW_CHANNEL_SINE_PERIOD          8u
#define DW_CHANNEL_SINE_LINE_SELECT     0u


static void initPWM(void);
static void initTrigMux(void);
static void initDMA(void);

#define PERIOD_SIZE 10U

//static uint16_t periods[PERIOD_SIZE] = {78U, 39U, 45U, 76U, 23U, 103U, 23U, 76U, 45U, 39U};   // sine 128kHz
static uint16_t periods[PERIOD_SIZE] = {182U, 90U, 105U, 177U, 53U, 241U, 53U, 177U, 105U, 89U}; // sine 55kHz

#define LINE_SELECT_SIZE 2U
static uint16_t lineSelects[LINE_SELECT_SIZE] = {0x12u, 0x21u};


/***************************** Pin cfg ****************************************/
cy_stc_gpio_pin_config_t pin_cfg1 = 
{
    .outVal    = 0x01u,
    .driveMode = CY_GPIO_DM_STRONG,
    .hsiom     = TCPWM_LINEx_MUX ,
    .intEdge   = 0u,
    .intMask   = 0u,
    .vtrip     = 0u,
    .slewRate  = 0u,
    .driveSel  = 0u,
    .vregEn    = 0u,
    .ibufMode  = 0u,
    .vtripSel  = 0u,
    .vrefSel   = 0u,
    .vohSel    = 0u,
};

cy_stc_gpio_pin_config_t pin_cfg2 = 
{
    .outVal    = 0x01u,
    .driveMode = CY_GPIO_DM_STRONG,
    .hsiom     = TCPWM_LINE_COMPLx_MUX,
    .intEdge   = 0u,
    .intMask   = 0u,
    .vtrip     = 0u,
    .slewRate  = 0u,
    .driveSel  = 0u,
    .vregEn    = 0u,
    .ibufMode  = 0u,
    .vtripSel  = 0u,
    .vrefSel   = 0u,
    .vohSel    = 0u,
};
/******************************************************************************/



/********************** PWM cfg ***********************************************/
cy_stc_tcpwm_pwm_config_t shePWM =
{
    .pwmMode            = CY_TCPWM_PWM_MODE_PWM,
    .clockPrescaler     = CY_TCPWM_PWM_PRESCALER_DIVBY_1,
    .debug_pause        = 1uL,
    .Cc0MatchMode       = CY_TCPWM_PWM_TR_CTRL2_NO_CHANGE,
    .OverflowMode       = CY_TCPWM_PWM_TR_CTRL2_INVERT,
    .UnderflowMode      = CY_TCPWM_PWM_TR_CTRL2_NO_CHANGE,
    .Cc1MatchMode       = CY_TCPWM_PWM_TR_CTRL2_NO_CHANGE,
    .deadTime           = 0uL,
    .deadTimeComp       = 0uL,
    .runMode            = CY_TCPWM_PWM_CONTINUOUS,
    .period             = 100uL,
    .period_buff        = 0uL,
    .enablePeriodSwap   = false,
    .compare0           = 0uL,
    .compare1           = 0uL,
    .enableCompare0Swap = false,
    .enableCompare1Swap = false,
    .interruptSources   = 0uL,
    .invertPWMOut       = 0uL,
    .invertPWMOutN      = 0uL,
    .killMode           = CY_TCPWM_PWM_NOT_STOP_ON_KILL , //CY_TCPWM_PWM_ASYNC_KILL,
    .switchInputMode    = 3uL,
    .switchInput        = 0uL,
    .reloadInputMode    = 3uL,
    .reloadInput        = 0uL,
    .startInputMode     = 3uL,
    .startInput         = 0uL,
    .kill0InputMode     = 3uL,
    .kill0Input         = 0uL,
    .kill1InputMode     = 3uL,
    .kill1Input         = 0uL,
    .countInputMode     = 3uL,
    .countInput         = 1uL,
};
/*******************************************************************************/



/********************** DMA cfg ***********************************************/
static cy_stc_pdma_descr_t stcDescr0 = {0u};
const cy_stc_pdma_chnl_config_t chnl0Config = {
    .PDMA_Descriptor = &stcDescr0,
    .preemptable     = 0,
    .priority        = 1,
    .enable          = 1, /*enabled after initialization*/
};
static cy_stc_pdma_descr_config_t stcDmaDescr0Config = {
    .deact           = 0, /*Do not wait for trigger de-activation*/
    .intrType        = CY_PDMA_INTR_1ELEMENT_CMPLT,
    .trigoutType     = CY_PDMA_TRIGOUT_X_LOOP_CMPLT,
    .chStateAtCmplt  = CY_PDMA_CH_ENABLED,
    .triginType      = CY_PDMA_TRIGIN_1ELEMENT,
    .dataSize        = CY_PDMA_HALFWORD,
    .srcTxfrSize     = 0,  /*= dataSize*/
    .destTxfrSize    = 1, /*= dataSize*/
    .descrType       = CY_PDMA_1D_TRANSFER,
    .srcAddr         = periods,
    .destAddr        = (uint32_t *)&TCPWM0->GRP[1U].CNT[6U].unPERIOD.u32Register,
    .srcXincr        = 1,
    .destXincr       = 0,
    .xCount          = PERIOD_SIZE,
    .srcYincr        = 0,
    .destYincr       = 0,
    .yCount          = 0,
    .descrNext       = &stcDescr0,
};

static cy_stc_pdma_descr_t stcDescr1 = {0u};
const cy_stc_pdma_chnl_config_t chnl1Config = {
    .PDMA_Descriptor = &stcDescr1,
    .preemptable     = 0,
    .priority        = 1,
    .enable          = 1, /*enabled after initialization*/
};
static cy_stc_pdma_descr_config_t stcDmaDescr1Config = {
    .deact           = 0, /*Do not wait for trigger de-activation*/
    .intrType        = CY_PDMA_INTR_1ELEMENT_CMPLT,
    .trigoutType     = CY_PDMA_TRIGIN_1ELEMENT,
    .chStateAtCmplt  = CY_PDMA_CH_ENABLED,
    .triginType      = CY_PDMA_TRIGIN_1ELEMENT,
    .dataSize        = CY_PDMA_HALFWORD,
    .srcTxfrSize     = 0,  /*= dataSize*/
    .destTxfrSize    = 1, /*= dataSize*/
    .descrType       = CY_PDMA_1D_TRANSFER,
    .srcAddr         = lineSelects,
    .destAddr        = (uint32_t *)&TCPWM0->GRP[1U].CNT[6U].unLINE_SEL.u32Register,
    .srcXincr        = 1,
    .destXincr       = 0,
    .xCount          = LINE_SELECT_SIZE,
    .srcYincr        = 0,
    .destYincr       = 0,
    .yCount          = 0,
    .descrNext       = &stcDescr1,
};

/*******************************************************************************/

void initSine(void)
{
  
  Cy_Tcpwm_Pwm_DeInit(TCPWM0_GRPx_CNTx_PWM );
   Cy_PDMA_Disable(DW0);
    initTrigMux();
    initPWM();
    initDMA();
    
    
    
    
   
}

static void initPWM(void)
{
    /*--------------------------------*/
    /* Clock Configuration for TCPWMs */
    /*--------------------------------*/
    Cy_SysClk_PeriphAssignDivider(PCLK_TCPWM0_CLOCKSx_PWM, (cy_en_divider_types_t)CY_SYSCLK_DIV_16_BIT, TCPWM_PERI_CLK_DIVIDER_NO_PWM);
    Cy_SysClk_PeriphSetDivider((cy_en_divider_types_t)CY_SYSCLK_DIV_16_BIT, TCPWM_PERI_CLK_DIVIDER_NO_PWM, 0u); // Divider 0 --> 160MHz / (1) = 160MHz
    Cy_SysClk_PeriphEnableDivider((cy_en_divider_types_t)CY_SYSCLK_DIV_16_BIT, TCPWM_PERI_CLK_DIVIDER_NO_PWM);

    /*------------------------------*/
    /* Port Configuration for TCPWM */
    /*------------------------------*/
    Cy_GPIO_Pin_Init(TCPWM_LINEx_PORT, TCPWM_LINEx_PIN, &pin_cfg1);
    Cy_GPIO_Pin_Init(TCPWM_LINE_COMPLx_PORT, TCPWM_LINE_COMPLx_PIN, &pin_cfg2);
    
    //Cy_Tcpwm_Pwm_DeInit(TCPWM0_GRPx_CNTx_PWM );

    /* Initialize TCPWM0_GRPx_CNTx_PWM_PR as PWM Mode & Enable */
    Cy_Tcpwm_Pwm_Init(TCPWM0_GRPx_CNTx_PWM, &shePWM);

    /* Counting mode */
    TCPWM0->GRP[1].CNT[6].unCTRL.stcField.u2UP_DOWN_MODE = TIMER_CTRL_COUNT_UP;

    // Initial values
    TCPWM0->GRP[1].CNT[6].unCC0.u32Register = 0u;
    TCPWM0->GRP[1].CNT[6].unPERIOD.u32Register = periods[0u];
    TCPWM0->GRP[1].CNT[6].unLINE_SEL.u32Register = lineSelects[1u];

    // Trigger out settings
    TCPWM0->GRP[1].CNT[6].unTR_OUT_SEL.u32Register = 0x00u;
    TCPWM0->GRP[1].CNT[6].unTR_OUT_SEL.stcField.u3OUT0 = TIMER_TR_OUT_ON_OVERFLOW;
    TCPWM0->GRP[1].CNT[6].unTR_OUT_SEL.stcField.u3OUT1 = TIMER_TR_OUT_DISABLED;

    // Invert outputs
    TCPWM0->GRP[1].CNT[6].unCTRL.stcField.u2QUAD_ENCODING_MODE = TIMER_CTRL_INV_BOTH_OUT;

    Cy_Tcpwm_Pwm_Enable(TCPWM0_GRPx_CNTx_PWM);
    Cy_Tcpwm_TriggerStart(TCPWM0_GRPx_CNTx_PWM);
}

static void initTrigMux(void)
{
    /* PWM_TRIGGER -> DMA (DMA transfers Period value)*/
    Cy_TrigMux_Connect(TRIG_IN_MUX_3_TCPWM_16M_TR_OUT06, TRIG_OUT_MUX_3_PDMA0_TR_IN8, 0, TRIGGER_TYPE_EDGE, 0u);

    /* P_DMA -> P_DMA (DMA transfers Line Select value)*/
    Cy_TrigMux_Connect(TRIG_IN_MUX_0_PDMA0_TR_OUT8, TRIG_OUT_MUX_0_PDMA0_TR_IN0, 0, TRIGGER_TYPE_EDGE, 0u);
}

static void initDMA(void)
{
    Cy_PDMA_Disable(DW0);
    
    //Channel 0 - Period
    Cy_PDMA_Descr_DeInit(&stcDescr0);
    Cy_PDMA_Descr_Init(&stcDescr0, &stcDmaDescr0Config);
    Cy_PDMA_Chnl_DeInit(DW0, DW_CHANNEL_SINE_PERIOD);
    Cy_PDMA_Chnl_Init(DW0, DW_CHANNEL_SINE_PERIOD, (const cy_stc_pdma_chnl_config_t *)&chnl0Config);
    Cy_PDMA_Chnl_Enable(DW0, DW_CHANNEL_SINE_PERIOD);

    //Channel 1 - Line select
    Cy_PDMA_Descr_DeInit(&stcDescr1);
    Cy_PDMA_Descr_Init(&stcDescr1, &stcDmaDescr1Config);
    Cy_PDMA_Chnl_DeInit(DW0, DW_CHANNEL_SINE_LINE_SELECT);
    Cy_PDMA_Chnl_Init(DW0, DW_CHANNEL_SINE_LINE_SELECT, (const cy_stc_pdma_chnl_config_t *)&chnl1Config);
    Cy_PDMA_Chnl_Enable(DW0, DW_CHANNEL_SINE_LINE_SELECT);

    Cy_PDMA_Enable(DW0);
}

